package nz.gen.wellington.screenscrapers.auckland;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import nz.co.searchwellington.utils.UrlFilters;
import nz.gen.wellington.screenscrapers.Newsitem;
import nz.gen.wellington.screenscrapers.NewsitemImpl;
import nz.gen.wellington.screenscrapers.ScreenScraperImpl;

import org.apache.log4j.Logger;
import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.Tag;
import org.htmlparser.filters.TagNameFilter;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;


/**
 * Extracts news items from Manukau City Councils latest news page.
 * org.htmlparser seems to have trouble with this markup; hence the string processing.
 * 
 * @author tony
 *
 */
public class ManukauCityCouncilScraper extends ScreenScraperImpl {
    
    
    private static final String MANUKAU_DATE_FORMAT = "dd MMMM yyyy";
    Logger log = Logger.getLogger(ManukauCityCouncilScraper.class);

    

    public String getPageURL() {
        return "http://www.manukau.govt.nz/latest.htm";
    }

    
    
    public List <Newsitem> parseContent(String content) {
        List <Newsitem> newsitems = new ArrayList <Newsitem> ();
        Parser parser = new Parser();

        try {
            parser.setInputHTML(content);

            List blocks = extractBlocks(content);
            
            for (int i=0; i< blocks.size(); i++) {
                Newsitem newsitem = extractNewsitemFromBlock((String) blocks.get(i));
                newsitem.flattenLoudCaps();
                
                if (validNewsitem(newsitem)) {
                    newsitems.add(newsitem);
                } else {
                    log.warn("Invalid newsitem detected; dropping.");
                }
            }

        } catch (ParserException e) {
            log.error("ParserException: " + e.getMessage());
        }
        return newsitems;
    }

    
    
   
    private Newsitem extractNewsitemFromBlock(String content) throws ParserException {

        Newsitem newsitem = new NewsitemImpl();

        Parser parser = new Parser();
        
        parser.setInputHTML(content);
        NodeFilter headingFilter = new TagNameFilter("H1");
        NodeList headingTags = parser.extractAllNodesThatMatch(headingFilter);
        if (headingTags.size() == 1) {
            Tag headingTag = (Tag) headingTags.elementAt(0);
            newsitem.setTitle(headingTag.toPlainTextString());            
        }
        
        parser.setInputHTML(content);
        NodeFilter linkFilter = new TagNameFilter("A");
        NodeList linkList = parser.extractAllNodesThatMatch(linkFilter);
      
        if (linkList.size() == 1) {
            Tag linkTag = (Tag) linkList.elementAt(0);            
            newsitem.setUrl("http://www.manukau.govt.nz/" + linkTag.getAttribute("HREF"));
        }
        
        List ptags = extractPtagsFromBlock(content);    
        if (ptags.size() > 1) {
            String dateString = UrlFilters.stripHtmlTags((String) ptags.get(0)); 
            newsitem.setDate(parseDate(dateString, MANUKAU_DATE_FORMAT));            
            newsitem.setDescription(UrlFilters.stripHtmlTags((String) ptags.get(1)));                        
        }
        
        return newsitem;
    }
    
   
    private List extractPtagsFromBlock(String block) {
        List tags = new ArrayList();
        
        Pattern p = Pattern.compile("<p>.*?</p>", Pattern.DOTALL);
        Matcher m = p.matcher(block);
        while (m.find()) {
            tags.add(m.group());
        }        
        return tags;        
    }
    
    private boolean validNewsitem(Newsitem newsitem) {
        return newsitem != null && newsitem.getTitle() != null && newsitem.getUrl() != null && newsitem.getDate() != null && newsitem.getDescription() != null;
    }
    

    
    private List extractBlocks(String content) {
        ArrayList blocks = new ArrayList();
        
        String[] splits = content.split("<hr size=\"1\">");
        for (int i=0; i < splits.length; i++) {
            String block = cleanBlock(splits[i]);
            if (block != null) {
                blocks.add(block);
            }
        }        
        return blocks;
    }

    
    private String cleanBlock(String block) {

        String[] splits = block.split("<h1>");
        if (splits.length == 2) {
            return stripHtmlAlign(stripHtmlDivTags("<h1>" + splits[1]));
        }
        
        return null;
    }
    
    
    public static String stripHtmlDivTags(String html) {
        Pattern p = Pattern.compile("</?div.*?>");
        Matcher m = p.matcher(html);
        return m.replaceAll("");
    }
  

    public static String stripHtmlAlign(String html) {
        Pattern p = Pattern.compile(" align=\"right\"");
        Matcher m = p.matcher(html);
        return m.replaceAll("");
    }
    
}
