package nz.gen.wellington.screenscrapers.twickenham;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import nz.co.searchwellington.utils.CachingHttpFetcher;
import nz.gen.wellington.screenscrapers.Newsitem;
import nz.gen.wellington.screenscrapers.NewsitemImpl;
import nz.gen.wellington.screenscrapers.ResolvableScreenScraper;
import nz.gen.wellington.screenscrapers.ScreenScraperImpl;

import org.apache.log4j.Logger;
import org.htmlparser.Node;
import org.htmlparser.Tag;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;
import org.joda.time.DateTime;

public class LBRUTPlanningApplicationsScraper extends ScreenScraperImpl implements ResolvableScreenScraper {
	
	private static final int MAX_ITEMS = 15;

	private final Logger log = Logger.getLogger(LBRUTPlanningApplicationsScraper.class);

	private static final String SEARCH_APPLICATIONS_URL = "http://www2.richmond.gov.uk/PlanData2/planning_summary.aspx";

	private CachingHttpFetcher httpFetcher;
	
	public LBRUTPlanningApplicationsScraper(CachingHttpFetcher httpFetcher) {
		this.httpFetcher = httpFetcher;
	}

	public String getPageURL() {		
		final DateTime now = new DateTime();
		final DateTime oneWeekAgo = new DateTime().minusWeeks(1);		
		return SEARCH_APPLICATIONS_URL + "?strWeekListType=SRCH&strDecFrom=&strDecTo=&strRecFrom=" + oneWeekAgo.toString("dd-MMM-yyyy") + "&strRecTo=" + now.toString("dd-MMM-yyyy") + "&strWard=ALL&strAppTyp=ALL&strWardTxt=All%20Wards&strAppTypTxt=ALL%20-%20all%20applications&strAppStat=ALL&strAppStatTxt=All%20Applications&strStreet=ALL&strStreetTxt=All%20Streets&strLimit=50&strOrder=REC_DEC&strOrderTxt=Most%20recently%20received%20first";
	}
	
	@Override
	public String getName() {
		return "LBRUT planning applications";
	}

	public String getCharacterEncoding() {
		return "UTF-8";
	}
	
	public List<Newsitem> parseContent(String content) throws Exception {
		List<Newsitem> newsitems = new ArrayList<Newsitem>();
		
		final Tag mainTable = extractTagByClassName(content, "blueCellBorders");				
		NodeList tableRows = extractTagsByName(mainTable.toHtml(), "tr");
		int size = tableRows.size();
		if (size > MAX_ITEMS) {
			size = MAX_ITEMS;
		}
		for (int i = 1; i < size; i++) {
         	Newsitem newsitem = new NewsitemImpl();
            Tag tableRow = (Tag) tableRows.elementAt(i);
            
            Node headingCell = tableRow.getFirstChild().getNextSibling();			
			newsitem.setTitle(headingCell.toPlainTextString().trim().replace("\r", ", "));
			
			Node applicationNumberCell = headingCell.getNextSibling().getNextSibling();
			final String applicationNumber = applicationNumberCell.toPlainTextString().trim();
			newsitem.setUrl("http://www2.richmond.gov.uk/PlanData2/Planning_CaseNo.aspx?strCASENO=" + applicationNumber);

			Node descriptionCell = applicationNumberCell.getNextSibling().getNextSibling();
			newsitem.setDescription(descriptionCell.toPlainTextString().trim());

			log.info("Fetching details of application: " + newsitem.getUrl());
			final String applicationDetailsPage = httpFetcher.httpFetch(newsitem.getUrl(), getCharacterEncoding(), null, null);
			if (applicationDetailsPage != null) {
				newsitem.setDate(getApplicationDateFromDetailsPage(applicationDetailsPage));
				setApplicationDateLocation(applicationDetailsPage, newsitem);
				setProposalText(applicationDetailsPage, newsitem);				
			}
									
			newsitems.add(newsitem);			
        }
		return newsitems;
	}


	public String getUrlStub() {
		return "lbrut-planning";
	}

	protected Date getApplicationDateFromDetailsPage(String content) throws ParserException {
		Tag applicationDateListElement = extractTagById(content, "cd_ap_stage1");
		Tag dateSpan = extractTagByClassName(applicationDateListElement.toHtml(), "date");
		String dateString = dateSpan.toPlainTextString();
		return parseDate(dateString, "dd/MM/yyyy");		
	}
	
	private void setProposalText(String content, Newsitem newsitem) throws ParserException {
		Tag proposalDescription = extractTagById(content, "ctl00_PageContent_lbl_Proposal");
		newsitem.setDescription(proposalDescription.toPlainTextString());			
	}

	protected void setApplicationDateLocation(String content, Newsitem newsitem) throws ParserException {
		Tag mapLink = extractTagById(content, "amap");
		if (mapLink != null) {
			String href = mapLink.getAttribute("href");
			href = href.replace("http://maps.google.com/?q=", "");
			String[] numbers = href.split("&")[0].split(",");
			
			BigDecimal latitude = new BigDecimal(numbers[0]);
			BigDecimal longitude = new BigDecimal(numbers[1]);
			
			newsitem.setLatitude(latitude);
			newsitem.setLongitude(longitude);
		}			
	}
	
}
